-- textWin.e
-- allows setting of text windows, or viewports.
-- redefines printf(), puts(), clear_screen()
-- j. craig gilbert
-- bitnaut@soulsteam.com
-----------------------------------------------------------------------------
-- GLOBAL VARIABLES:
-----------------------------------------------------------------------------
-- gCurrentTextVideoPage - defaults to zero; you should change it if you
--                         are using a different video page in text mode
-----------------------------------------------------------------------------
-- GLOBAL ROUTINES:
-----------------------------------------------------------------------------
-- procedure setTextVidAddress()
    -- updates  the module variable textVidAddress to reflect the
    -- correct text video memory address
-- function getTextVidAddress()
    -- returns the current text video memory address
    --
-- function getCursor()
    -- get current cursor shape, suitable to feed to cursor()
    --
-- procedure scrollTextWin(integer lines, integer upLeftX, integer upLeftY, integer lowRightX,integer lowRightY)
    -- scroll text window; expects Euphoria-style text coordinates (1-80,1-25)
    --
-- procedure setTextWin(integer upLeftX, integer upLeftY, integer lowRightX, integer lowRightY)
    -- set the dimensions/location of the text window
    --
-- procedure puts(integer fn, sequence s)
    -- reproduces the behaviour of puts() as appropriate for text windows
    --
-- procedure printf(integer fn,sequence fmt, sequence params)
    -- reproduces the behaviour of printf() as appropriate for text windows
    --
-- procedure clear_screen()
    -- reproduces the behaviour of clear_screen() as appropriate for text windows
    --
-- function textScreenMemAddress(integer col, integer row)
    -- gets the video memory address for a given (col,row) location
    --
-- procedure horizTextLine(integer char, integer leftX, integer rightX, integer Y)
    -- draw a horizontal line using the given character
    --
-- procedure vertTextLine(integer char, integer topY, integer bottomY, integer X)
    -- draw a vertical line using the given character
    --
-- procedure drawTextBox(integer char, integer upLeftX,integer upLeftY,
    -- draw a simple box on the screen using the given character
    --
-----------------------------------------------------------------------------
--  R O U T I N E S
-----------------------------------------------------------------------------
include machine.e
include graphics.e
--
-- change this if you fool around with displaying different video pages
-- in text mode
--
global integer gCurrentTextVideoPage
gCurrentTextVideoPage = 0
atom textVidAddress -- memory address, start of text video memory
-----------------------------------------------------------------------------
global procedure setTextVidAddress()
-- updates  the module variable textVidAddress to reflect the
-- correct text video memory address
sequence vidConfig
    --get vid address
    vidConfig = video_config()
    if vidConfig[VC_MODE] = 7 then -- monochrome
        textVidAddress = #B0000
    else -- better be non monochrome text mode if you're using this file . .
        textVidAddress = #B8000
    end if
end procedure
setTextVidAddress()
-----------------------------------------------------------------------------
global function getTextVidAddress()
-- returns the current text video memory address
--
    setTextVidAddress() -- update it in case vid mode has changed
    return textVidAddress
end function
-----------------------------------------------------------------------------
global function getCursor()
-- get current cursor shape, suitable to feed to cursor()

sequence regs, dum

    regs = repeat(0,10)
    regs[REG_AX] = #0300
    regs[REG_BX] = gCurrentTextVideoPage * #100

    dum = dos_interrupt(#10,regs)

    return dum[REG_CX]
end function
-----------------------------------------------------------------------------
function currentTextAttribute()
-- this is a real pain in the butt.
sequence textPos, firstCharAndAttr, newCharAndAttr
integer oldCursor

    -- save char and attribute at 1,1
    firstCharAndAttr = peek({textVidAddress,2})
    -- save current text pos
    textPos = get_position()
    -- print char at 1,1 using puts() so current color
    -- will be used
    oldCursor = getCursor()
    cursor(NO_CURSOR)
    position(1,1)
    puts(1,firstCharAndAttr[1])
    -- get attribute at 1,1
    newCharAndAttr = peek({textVidAddress,2})
    -- restore original char and attribute at 1,1
    poke(textVidAddress+1,firstCharAndAttr[2])
    -- restore original text pos
    position(textPos[1],textPos[2])
    cursor(oldCursor)

    return newCharAndAttr[2] -- all we want is the attribute
end function
-----------------------------------------------------------------------------
global procedure scrollTextWin(integer lines, integer upLeftX, integer upLeftY,
                                   integer lowRightX,integer lowRightY)
-- scroll text window; expects Euphoria-style text coordinates (1-80,1-25)
sequence regs, dum

    regs = repeat(0,10)
    regs[REG_BX] = currentTextAttribute() * #100
    regs[REG_DX] = (lowRightY-1) * #100 + (lowRightX-1)
    regs[REG_CX] = (upLeftY-1) * #100 + (upLeftX-1)
    regs[REG_AX] = #0600 + lines

    dum = dos_interrupt(#10,regs)
end procedure
-----------------------------------------------------------------------------
-- module-level vars
sequence currentTextWindow
constant LEFTCOL = 1, TOPROW = 2, RIGHTCOL = 3, BOTROW = 4
currentTextWindow = {1,1,80,25} --default standard text win
global procedure setTextWin(integer upLeftX, integer upLeftY,
                     integer lowRightX, integer lowRightY)
-- set the dimensions/location of the text window
    currentTextWindow = {upLeftX, upLeftY, lowRightX, lowRightY}
    position(upLeftY, upLeftX)
end procedure
-----------------------------------------------------------------------------
procedure origPrintf(integer fn,sequence fmt, sequence params)
    printf(fn,fmt,params)
end procedure
-----------------------------------------------------------------------------
procedure origPuts(integer fn, sequence s)
    puts(fn,s)
end procedure
-----------------------------------------------------------------------------
global procedure puts(integer fn, sequence s)
-- reproduces the behaviour of puts() as appropriate for text windows
integer windowWidth, windowHeight, ch
sequence curPos

    if fn != 1 then -- not screen, no need for anything special
        origPuts(fn,s)
    else
        windowWidth = currentTextWindow[RIGHTCOL] - currentTextWindow[LEFTCOL] + 1
        windowHeight = currentTextWindow[BOTROW] - currentTextWindow[TOPROW] + 1
        if windowWidth = 80 and windowHeight = 25 then -- standard screen
            origPuts(fn,s)
            return
        end if

        ch = 1
        while ch <= length(s) do
            curPos = get_position()
            if s[ch] = 0 then -- Euphoria won't print a string past a nul
                exit
            elsif s[ch] = '\t' then -- tab = 7 spaces in DOS
                -- insert spaces and let normal processing handle them
                s = s[1..ch] & repeat(' ',7) & s[(ch+1)..length(s)]
            elsif s[ch] = '\n' then -- newline (Euphoria translates to lf/cr)
                if curPos[1] = currentTextWindow[BOTROW] then
                    scrollTextWin(1,currentTextWindow[LEFTCOL],currentTextWindow[TOPROW],
                                    currentTextWindow[RIGHTCOL],currentTextWindow[BOTROW])
                    position(curPos[1],currentTextWindow[LEFTCOL])
                else
                    position(curPos[1]+1,currentTextWindow[LEFTCOL])
                end if
            elsif s[ch] = '\r' then -- carriage return
                position(curPos[1],currentTextWindow[LEFTCOL])
            else
                origPuts(1,{s[ch]})
                if curPos[2] = currentTextWindow[RIGHTCOL] then
                    if curPos[1] = currentTextWindow[BOTROW] then
                        scrollTextWin(1,currentTextWindow[LEFTCOL],currentTextWindow[TOPROW],
                                        currentTextWindow[RIGHTCOL],currentTextWindow[BOTROW])

                        position(currentTextWindow[BOTROW],currentTextWindow[LEFTCOL])
                    else
                        position(curPos[1]+1,currentTextWindow[LEFTCOL])
                    end if
                end if
            end if
            ch = ch + 1
        end while

    end if
end procedure
-----------------------------------------------------------------------------
global procedure printf(integer fn,sequence fmt, sequence params)
-- reproduces the behaviour of printf() as appropriate for text windows
    if fn != 1 then -- not screen, no need for anything special
        origPrintf(fn,fmt,params)
    else
        puts(1,sprintf(fmt,params))
    end if
end procedure
-----------------------------------------------------------------------------
global procedure clear_screen()
-- reproduces the behaviour of clear_screen() as appropriate for text windows
    scrollTextWin(currentTextWindow[4]-currentTextWindow[2]+1,
              currentTextWindow[1],currentTextWindow[2],
              currentTextWindow[3],currentTextWindow[4])
    position(currentTextWindow[2],currentTextWindow[1])
end procedure
-----------------------------------------------------------------------------
global function textScreenMemAddress(integer col, integer row)
-- gets the video memory address for a given (col,row) location
    return (textVidAddress + (row-1)*160 + (col-1)*2)
end function
-----------------------------------------------------------------------------
global procedure horizTextLine(integer char, integer leftX, integer rightX, integer Y)
-- draw a horizontal line using the given character
sequence line
integer attr
    line = repeat(char, (rightX-leftX+1)*2)
    attr = currentTextAttribute()
    for attrLoc = 2 to length(line) by 2 do
        line[attrLoc] = attr
    end for
    poke( textScreenMemAddress(leftX,Y), line)
end procedure
-----------------------------------------------------------------------------
global procedure vertTextLine(integer char, integer topY, integer bottomY, integer X)
-- draw a vertical line using the given character
integer attr
    attr = currentTextAttribute()
    for addr = textScreenMemAddress(X,topY) to textScreenMemAddress(X,bottomY) by 160 do
        poke(addr, {char,attr})
    end for
end procedure
-----------------------------------------------------------------------------
global procedure drawTextBox(integer char, integer upLeftX,integer upLeftY,
                                           integer lowRightX, integer lowRightY)
-- draw a simple box on the screen using the given character
    -- top
    horizTextLine(char, upLeftX, lowRightX, upLeftY)
    -- right side
    vertTextLine(char, upLeftY+1, lowRightY-1, lowRightX)
    -- bottom

    horizTextLine(char, upLeftX, lowRightX, lowRightY)
    -- left side
    vertTextLine(char, upLeftY+1, lowRightY-1, upLeftX)
end procedure
-----------------------------------------------------------------------------
